@@ -2,7 +2,7 @@
  *
  * Interface and conversion functions for GMP.
  *
- * $Id: bignum.h,v 1.16 1999/11/28 18:01:36 nisse Exp $ */
+ * $Id: bignum.h,v 1.17 2000/04/23 17:48:45 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -41,8 +41,8 @@
 #endif
 #endif
 
-void bignum_parse_s(mpz_t n, UINT32 length, UINT8 *data);
-void bignum_parse_u(mpz_t n, UINT32 length, UINT8 *data);
+void bignum_parse_s(mpz_t n, UINT32 length, const UINT8 *data);
+void bignum_parse_u(mpz_t n, UINT32 length, const UINT8 *data);
 
 UINT32 bignum_format_s(const mpz_t n, UINT8 *data);
 UINT32 bignum_format_s_length(const mpz_t n);
