@@ -1,6 +1,6 @@
 /* client.c
  *
- * $Id: client.c,v 1.93 2000/04/23 17:48:47 nisse Exp $ */
+ * $Id: client.c,v 1.94 2000/05/21 21:14:40 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -285,7 +285,7 @@ do_exit_signal(struct channel_request *c,
 
       signal = signal_network_to_local(signal);
 
-      werror("%us", length, msg);
+      werror("%ups", length, msg);
       werror("Remote process was killed by %z.%z\n",
 	     signal ? STRSIGNAL(signal) : "an unknown signal",
 	     core ? "(core dumped remotely)\n": "");
