@@ -1,6 +1,6 @@
 /* dsa.c
  *
- * $Id: dsa.c,v 1.13 2000/05/06 11:17:32 nisse Exp $
+ * $Id: dsa.c,v 1.15 2000/05/22 20:40:56 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -101,11 +101,11 @@ static void generic_dsa_sign(struct dsa_signer *closure,
   /* Compute r = (g^k (mod p)) (mod q) */
   mpz_powm(r, closure->public.g, k, closure->public.p);
 
-  debug("do_dsa_sign, group element: %xn\n", r);
+  debug("generic_dsa_sign, group element: %xn\n", r);
   
   mpz_fdiv_r(r, r, closure->public.q);
 
-  debug("do_dsa_sign, r: %xn\n", r);
+  debug("generic_dsa_sign, r: %xn\n", r);
 
   /* Compute hash */
   dsa_hash(tmp, length, msg);
@@ -113,7 +113,7 @@ static void generic_dsa_sign(struct dsa_signer *closure,
   /* Compute k^-1 (mod q) */
   if (!mpz_invert(k, k, closure->public.q))
     {
-      fatal("do_dsa_sign: k non-invertible\n");
+      fatal("generic_dsa_sign: k non-invertible\n");
     }
 
   /* Compute signature s = k^-1(h + ar) (mod q) */
@@ -161,6 +161,9 @@ do_dsa_sign(struct signer *c,
       
   /* Build signature */
   buf_length = dsa_blob_length(r, s);
+
+  /* NOTE: draft-ietf-secsh-transport-X.txt (x <= 07) uses an extra
+   * length field, which should be removed in the next version. */
   signature = ssh_format("%a%r", ATOM_SSH_DSS, buf_length * 2, &p);
   dsa_blob_write(r, s, buf_length, p);
   
@@ -343,6 +346,9 @@ do_dsa_verify(struct verifier *c,
   int atom;
   mpz_t r, s;
 
+  /* NOTE: draft-ietf-secsh-transport-X.txt (x <= 07) uses an extra
+   * length field, which should be removed in the next version. */
+
   UINT32 buf_length;
   const UINT8 *buf;
   
