@@ -2,7 +2,7 @@
  *
  * Simple mark&sweep garbage collector.
  *
- * $Id: gc.c,v 1.12 1999/12/20 20:40:12 nisse Exp $ */
+ * $Id: gc.c,v 1.13 2000/04/23 16:59:46 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -155,20 +155,41 @@ void gc_register(struct lsh_object *o)
   sanity_check_object_list();
 }
 
-/* FIXME: This function should really deallocate and forget the object
- * early. But we keep it until the next gc, in order to catch any
- * references to killed objects. */
+/* NOTE: If the object is close to the start of the object list, it is
+ * deallocated and forgotten immedietely. If the object is not found,
+ * we don't search the entire list, but instead defer deallocation to
+ * gc_sweep(). */
 void gc_kill(struct lsh_object *o)
 {
   sanity_check_object_list();
 
   if (!o)
     return;
-  
+
   assert(!o->dead);
 
   o->dead = 1;
 
+  debug("gc_kill: Killing object of type %z.\n",
+	o->isa ? o->isa->name : "UNKNOWN");
+  
+  if (o == all_objects)
+    {
+      struct lsh_class *class;
+      
+      debug("gc_kill:   Deallocating immediately.\n");
+
+      for (class = o->isa; class; class = class->super_class)
+	if (class->free_instance)
+	  FREE_INSTANCE(class, o);
+
+      all_objects = o->next;
+      number_of_objects--;
+      lsh_object_free(o);
+    }
+  else
+    debug("gc_kill:   Deferring deallocation to gc_sweep()\n");
+
   sanity_check_object_list();
 }
 
