@@ -2,7 +2,7 @@
  *
  *
  *
- * $Id: io.h,v 1.59 2000/04/21 22:29:38 nisse Exp $ */
+ * $Id: io.h,v 1.60 2000/05/27 12:55:30 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -349,4 +349,11 @@ io_read_file(struct io_backend *backend,
 	     const char *fname, 
 	     struct exception_handler *e);
 
+struct lsh_fd *
+io_read_user_file(struct io_backend *backend,
+		  const char *fname,
+		  uid_t uid, int secret,
+		  const struct exception **x,
+		  struct exception_handler *e);
+
 #endif /* LSH_IO_H_INCLUDED */
