@@ -1,6 +1,6 @@
 /* keyexchange.c
  *
- * $Id: keyexchange.c,v 1.63 2000/05/06 11:17:35 nisse Exp $ */
+ * $Id: keyexchange.c,v 1.65 2000/05/28 00:14:51 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -25,7 +25,9 @@
 
 #include "abstract_io.h"
 #include "alist.h"
+#include "command.h"
 #include "connection.h"
+#include "debug.h"
 #include "disconnect.h"
 #include "format.h"
 #include "parse.h"
@@ -69,7 +71,8 @@
 /* Arbitrary limit on list length */
 #define KEXINIT_MAX_ALGORITMS 47
 
-static struct kexinit *parse_kexinit(struct lsh_string *packet)
+static struct kexinit *
+parse_kexinit(struct lsh_string *packet)
 {
   NEW(kexinit, res);
   struct simple_buffer buffer;
@@ -125,7 +128,8 @@ static struct kexinit *parse_kexinit(struct lsh_string *packet)
   return res;
 }
 
-struct lsh_string *format_kex(struct kexinit *kex)
+struct lsh_string *
+format_kex(struct kexinit *kex)
 {
   return ssh_format("%c%ls%A%A%A%A%A%A%A%A%A%A%c%i",
 		    SSH_MSG_KEXINIT,
@@ -342,13 +346,14 @@ make_kexinit_handler(int type,
 
 #define IV_TYPE(t) ((t) + 4)
 
-static struct lsh_string *kex_make_key(struct hash_instance *secret,
-				       UINT32 key_length,
-				       int type,
-				       struct lsh_string *session_id)
+static struct lsh_string *
+kex_make_key(struct hash_instance *secret,
+	     UINT32 key_length,
+	     int type,
+	     struct lsh_string *session_id)
 {
   /* Indexed by the KEX_* values */
-  static /* const */ UINT8 tags[] = "CDEFAB";
+  static const UINT8 tags[] = "CDEFAB";
   
   struct lsh_string *key;
   struct hash_instance *hash;
@@ -423,10 +428,11 @@ static struct lsh_string *kex_make_key(struct hash_instance *secret,
   return key;
 }
   
-struct crypto_instance *kex_make_encrypt(struct hash_instance *secret,
-					 struct object_list *algorithms,
-					 int type,
-					 struct lsh_string *session_id)
+struct crypto_instance *
+kex_make_encrypt(struct hash_instance *secret,
+		 struct object_list *algorithms,
+		 int type,
+		 struct lsh_string *session_id)
 {
   CAST_SUBTYPE(crypto_algorithm, algorithm, LIST(algorithms)[type]);
     
@@ -455,10 +461,11 @@ struct crypto_instance *kex_make_encrypt(struct hash_instance *secret,
   return crypto;
 }
 
-struct crypto_instance *kex_make_decrypt(struct hash_instance *secret,
-					 struct object_list *algorithms,
-					 int type,
-					 struct lsh_string *session_id)
+struct crypto_instance *
+kex_make_decrypt(struct hash_instance *secret,
+		 struct object_list *algorithms,
+		 int type,
+		 struct lsh_string *session_id)
 {
   CAST_SUBTYPE(crypto_algorithm, algorithm, LIST(algorithms)[type]);
 
@@ -486,10 +493,11 @@ struct crypto_instance *kex_make_decrypt(struct hash_instance *secret,
   return crypto;
 }
 
-struct mac_instance *kex_make_mac(struct hash_instance *secret,
-				  struct object_list *algorithms,
-				  int type,
-				  struct lsh_string *session_id)
+struct mac_instance *
+kex_make_mac(struct hash_instance *secret,
+	     struct object_list *algorithms,
+	     int type,
+	     struct lsh_string *session_id)
 {
   CAST_SUBTYPE(mac_algorithm, algorithm, LIST(algorithms)[type]);
 
@@ -510,16 +518,18 @@ struct mac_instance *kex_make_mac(struct hash_instance *secret,
   return mac;
 }
 
-static struct compress_instance *kex_make_deflate(struct object_list *algorithms,
-						   int type)
+static struct compress_instance *
+kex_make_deflate(struct object_list *algorithms,
+		 int type)
 {
   CAST_SUBTYPE(compress_algorithm, algorithm, LIST(algorithms)[type]);
   
   return algorithm ? MAKE_DEFLATE(algorithm) : NULL;
 }
 
-static struct compress_instance *kex_make_inflate(struct object_list *algorithms,
-						  int type)
+static struct compress_instance *
+kex_make_inflate(struct object_list *algorithms,
+		 int type)
 {
   CAST_SUBTYPE(compress_algorithm, algorithm, LIST(algorithms)[type]);
 
@@ -625,13 +635,14 @@ do_make_simple_kexinit(struct make_kexinit *c)
   return kex;
 }
 
-struct make_kexinit *make_simple_kexinit(struct randomness *r,
-					 struct int_list *kex_algorithms,
-					 struct int_list *hostkey_algorithms,
-					 struct int_list *crypto_algorithms,
-					 struct int_list *mac_algorithms,
-					 struct int_list *compression_algorithms,
-					 struct int_list *languages)
+struct make_kexinit *
+make_simple_kexinit(struct randomness *r,
+		    struct int_list *kex_algorithms,
+		    struct int_list *hostkey_algorithms,
+		    struct int_list *crypto_algorithms,
+		    struct int_list *mac_algorithms,
+		    struct int_list *compression_algorithms,
+		    struct int_list *languages)
 {
   NEW(simple_kexinit, res);
 
@@ -658,9 +669,10 @@ struct make_kexinit *make_simple_kexinit(struct randomness *r,
        (algorithms object object_list)))
 */
 
-static int do_install(struct install_keys *c,
-		      struct ssh_connection *connection,
-		      struct hash_instance *secret)
+static int
+do_install(struct install_keys *c,
+	   struct ssh_connection *connection,
+	   struct hash_instance *secret)
 {
   CAST(install_new_keys, closure, c);
   struct crypto_instance *rec;
@@ -708,8 +720,9 @@ static int do_install(struct install_keys *c,
   return 1;
 }
 
-struct install_keys *make_install_new_keys(int is_server,
-					   struct object_list *algorithms)
+struct install_keys *
+make_install_new_keys(int is_server,
+		      struct object_list *algorithms)
 {
   NEW(install_new_keys, self);
 
@@ -723,20 +736,69 @@ struct install_keys *make_install_new_keys(int is_server,
 /* Returns a hash instance for generating various session keys. NOTE:
  * This mechanism changed in the transport-05 draft. Before this, the
  * exchange hash was not included at this point. */
-struct hash_instance *
+static struct hash_instance *
 kex_build_secret(struct hash_algorithm *H,
 		 struct lsh_string *exchange_hash,
-		 mpz_t K)
+		 struct lsh_string *secret)
 {
   struct hash_instance *hash = MAKE_HASH(H);
-  struct lsh_string *s = ssh_format("%n", K);
 
-  HASH_UPDATE(hash, s->length, s->data);
-  lsh_string_free(s);
+  HASH_UPDATE(hash, secret->length, secret->data);
   
   HASH_UPDATE(hash, exchange_hash->length, exchange_hash->data);
 
   return hash;
 }
 
+void
+keyexchange_finish(struct ssh_connection *connection,
+		   struct install_keys *install,
+		   struct hash_algorithm *H,
+		   struct lsh_string *exchange_hash,
+		   struct lsh_string *K)
+{
+  struct hash_instance *hash;
+  
+  /* Send a newkeys message, and install a handler for receiving the
+   * newkeys message. */
 
+  C_WRITE(connection, ssh_format("%c", SSH_MSG_NEWKEYS));
+
+  /* A hash instance initialized with the key, to be used for key
+   * generation */
+  hash = kex_build_secret(H, exchange_hash, K);
+  lsh_string_free(K);
+  
+  /* Record session id */
+  if (!connection->session_id)
+    connection->session_id = exchange_hash;
+  else
+    lsh_string_free(exchange_hash);
+  
+  if (!INSTALL_KEYS(install, connection, hash))
+    {
+      werror("Installing new keys failed. Hanging up.\n");
+      KILL(hash);
+
+      PROTOCOL_ERROR(connection->e, "Refusing to use weak key.");
+
+      return;
+    }
+
+  KILL(hash);
+
+  connection->kex_state = KEX_STATE_NEWKEYS;
+
+#if DATAFELLOWS_WORKAROUNDS
+  if (! (connection->peer_flags & PEER_SEND_NO_DEBUG))
+#endif
+    send_verbose(connection->write, "Key exchange successful!", 0);
+
+  if (connection->established)
+    {
+      struct command_continuation *c = connection->established;
+      connection->established = NULL;
+  
+      COMMAND_RETURN(c, connection);
+    }
+}
