@@ -2,7 +2,7 @@
  *
  *
  *
- * $Id: publickey_crypto.h,v 1.31 2000/04/23 17:48:51 nisse Exp $ */
+ * $Id: publickey_crypto.h,v 1.35 2000/05/28 21:15:03 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -44,9 +44,10 @@
        (private object signer)))
 */
 
-struct keypair *make_keypair(UINT32 type,
-			     struct lsh_string *public,
-			     struct signer *private);
+struct keypair *
+make_keypair(UINT32 type,
+	     struct lsh_string *public,
+	     struct signer *private);
 
 /* DSA definitions */
 /* GABA:
@@ -114,56 +115,75 @@ make_rsa_algorithm(struct hash_algorithm *hash,
 		   UINT32 prefix_length,
 		   const UINT8 *prefix);
 
-/* FIXME: Groups could use (meta)class methods */
-
 /* Groups. For now, assume that all group elements are represented by
  * bignums. */
 /* GABA:
    (class
-     (name group)
+     (name abstract_group)
      (vars
        (order bignum)
+       (generator bignum)
        ;; We should have a generator here, as we always work within some
        ;; cyclic subgroup.
-       (member method int "mpz_t x")
+       
+       ;; Checks if a bignum is in the correct range for being a group element. 
+       (range method int "mpz_t x")
        (invert method void "mpz_t res" "mpz_t x")
        (combine method void "mpz_t res" "mpz_t a" "mpz_t b")
        ; FIXME: Doesn't handle negative exponents
-       (power method void "mpz_t res" "mpz_t g" "mpz_t e")))
+       (power method void "mpz_t res" "mpz_t g" "mpz_t e")
+       (small_power method void "mpz_t res" "mpz_t g" "UINT32 e")))
 */
 
-#define GROUP_MEMBER(group, x) ((group)->member((group), (x)))
+#define GROUP_RANGE(group, x) ((group)->range((group), (x)))
 #define GROUP_INVERT(group, res, x) ((group)->invert((group), (res), (x)))
 #define GROUP_COMBINE(group, res, a, b) \
 ((group)->combine((group), (res), (a), (b)))
 #define GROUP_POWER(group, res, g, e) \
 ((group)->power((group), (res), (g), (e)))
+#define GROUP_SMALL_POWER(group, res, g, e) \
+((group)->small_power((group), (res), (g), (e)))
 
-struct group *make_zn(mpz_t p, mpz_t order);
+/* Groups */
+/* GABA:
+   (class
+     (name group_zn)
+     (super abstract_group)
+     (vars
+       (modulo bignum)))
+*/
 
+struct group_zn *
+make_zn(mpz_t p, mpz_t g, mpz_t order);
+
+void
+zn_ring_add(struct abstract_group *s,
+	    mpz_t res, mpz_t a, mpz_t b);
+void
+zn_ring_subtract(struct abstract_group *s,
+		 mpz_t res, mpz_t a, mpz_t b);
+
+struct abstract_group *
+make_ssh_group1(void);
 
 /* DH key exchange, with authentication */
 /* GABA:
    (class
-     (name diffie_hellman_method)
+     (name dh_method)
      (vars
-       (G object group)
-       (generator bignum)
+       (G object abstract_group)
        (H object hash_algorithm)
        (random object randomness)))
 */
 
-/* NOTE: Instances are never allocated on the heap by themselves. They
- * are always embedded in other objects. Therefore there's no object
- * header. */
-
 /* GABA:
    (struct
-     (name diffie_hellman_instance)
+     (name dh_instance)
      (vars
-       (method object diffie_hellman_method)
+       (method object dh_method)
        (e bignum)       ; Client value
-       (f bignum)       ; Server value 
+       (f bignum)       ; Server value
+       ;; FIXME: Move these somewhere else?
        (server_key string)
        (signature string)
        (secret bignum)  ; This side's secret exponent
@@ -173,42 +193,55 @@ struct group *make_zn(mpz_t p, mpz_t order);
 */
 
 /* Creates client message */
-struct lsh_string *dh_make_client_msg(struct diffie_hellman_instance *self);
+struct lsh_string *
+dh_make_client_msg(struct dh_instance *self);
 
 /* Receives client message */
-int dh_process_client_msg(struct diffie_hellman_instance *self,
-			  struct lsh_string *packet);
+int
+dh_process_client_msg(struct dh_instance *self,
+		      struct lsh_string *packet);
 
-#if 0
-/* Should be called with the kex_init messages, client's first */
-void dh_hash_update(struct diffie_hellman_instance *self,
-		    struct lsh_string *packet);
-#endif
+/* Includes more data to the exchange hash. */
+void
+dh_hash_update(struct dh_instance *self,
+	       struct lsh_string *s, int free);
 
 /* Generates server's secret exponent */
-void dh_make_server_secret(struct diffie_hellman_instance *self);
+void
+dh_make_server_secret(struct dh_instance *self);
 
 /* Creates server message */
-struct lsh_string *dh_make_server_msg(struct diffie_hellman_instance *self,
-				      struct signer *s);
+struct lsh_string *
+dh_make_server_msg(struct dh_instance *self,
+		   struct signer *s);
 
 /* Decodes server message, but does not verify its signature */
-int dh_process_server_msg(struct diffie_hellman_instance *self,
-			  struct lsh_string *packet);
+int
+dh_process_server_msg(struct dh_instance *self,
+		      struct lsh_string *packet);
 
 /* Verifies server's signature */
-int dh_verify_server_msg(struct diffie_hellman_instance *self,
-			 struct verifier *v);
+int
+dh_verify_server_msg(struct dh_instance *self,
+		     struct verifier *v);
 
-void dh_generate_secret(struct diffie_hellman_instance *self,
-			mpz_t r);
+void
+dh_generate_secret(struct dh_method *self,
+		   mpz_t r, mpz_t v);
 
-void dh_hash_digest(struct diffie_hellman_instance *self, UINT8 *digest);
+void
+dh_hash_digest(struct dh_instance *self);
 
-struct diffie_hellman_method *make_dh1(struct randomness *r);
+struct dh_method *
+make_dh(struct abstract_group *G, struct hash_algorithm *H,
+	struct randomness *r);
 
-void init_diffie_hellman_instance(struct diffie_hellman_method *m,
-				  struct diffie_hellman_instance *self,
-				  struct ssh_connection *c);
+struct dh_method *
+make_dh1(struct randomness *r);
+
+void
+init_dh_instance(struct dh_method *m,
+		 struct dh_instance *self,
+		 struct ssh_connection *c);
 
 #endif /* LSH_PUBLICKEY_CRYPTO_H_INCLUDED */
