@@ -40,169 +40,6 @@
 
 #include "server_userauth.c.x"
 
-#if 0
-/* ;; GABA:
-   (class
-     (name unix_user_db)
-     (super user_db)
-     (vars
-       (allow_root . int)))
-*/
-   
-/* NOTE: Calls functions using the disgusting convention of returning
- * pointers to static buffers. */
-static struct unix_user *
-do_lookup_user(struct user_db *s,
-	       struct lsh_string *name, int free)
-{
-  CAST(unix_user_db, self, s);
-  
-  struct passwd *passwd;
-
-  NEW(unix_user, res);
-
-  name = make_cstring(name, free);
-
-  if (!name)
-    {
-      KILL(res);
-      return NULL;
-    }
-  
-  if ((passwd = getpwnam(name->data))
-      /* Check for root login */
-      && (passwd->pw_uid || self->allow_root))
-    {      
-      res->uid = passwd->pw_uid;
-      res->gid = passwd->pw_gid;
-      res->super.name = name;
-  
-#if HAVE_GETSPNAM
-      /* FIXME: What's the most portable way to test for shadow passwords?
-       * A single character in the passwd field should cover most variants. */
-      if (passwd->pw_passwd && (strlen(passwd->pw_passwd) == 1))
-	{
-	  struct spwd *shadowpwd;
-    
-	  if (!(shadowpwd = getspnam(name->data)))
-	    goto fail;
-
-	  res->passwd = format_cstring(shadowpwd->sp_pwdp);
-	}
-      else
-#endif /* HAVE_GETSPNAM */
-	res->passwd = format_cstring(passwd->pw_passwd);
-
-      res->home = format_cstring(passwd->pw_dir);
-      res->shell = format_cstring(passwd->pw_shell);
-  
-      return res;
-    }
-  else
-    {
-    fail:
-      lsh_string_free(name);
-      KILL(res);
-      return NULL;
-    }
-}
-
-struct user_db *
-make_unix_user_db(int allow_root)
-{
-  NEW(unix_user_db, self);
-
-  self->super.lookup = do_lookup_user;
-  self->allow_root = allow_root;
-
-  return &self->super;
-}
-
-/* NOTE: Calls functions using the *disgusting* convention of returning
- * pointers to static buffers. */
-int verify_password(struct unix_user *user,
-		    struct lsh_string *password, int free)
-{
-  char *salt;
-  
-  if (!user->passwd || (user->passwd->length < 2) )
-    {
-      /* FIXME: How are accounts without passwords handled? */
-      lsh_string_free(password);
-      return 0;
-    }
-
-  /* Convert password to a NULL-terminated string */
-  password = make_cstring(password, free);
-
-  if (!password)
-    return 0;
-  
-  salt = user->passwd->data;
-
-  if (strcmp(crypt(password->data, salt), user->passwd->data))
-    {
-      /* Passwd doesn't match */
-      lsh_string_free(password);
-      return 0;
-    }
-
-  lsh_string_free(password);
-  return 1;
-}
-
-int change_uid(struct unix_user *user)
-{
-  /* NOTE: Error handling is crucial here. If we do something
-   * wrong, the server will think that the user is logged in
-   * under his or her user id, while in fact the process is
-   * still running as root. */
-  if (initgroups(user->super.name->data, user->gid) < 0)
-    {
-      werror("initgroups failed: %z\n", STRERROR(errno));
-      return 0;
-    }
-  if (setgid(user->gid) < 0)
-    {
-      werror("setgid failed: %z\n", STRERROR(errno));
-      return 0;
-    }
-  if (setuid(user->uid) < 0)
-    {
-      werror("setuid failed: %z\n", STRERROR(errno));
-      return 0;
-    }
-  return 1;
-}
-
-int change_dir(struct unix_user *user)
-{
-  /* Change to user's home directory. FIXME: If the server is running
-   * as the same user, perhaps it's better to use $HOME? */
-  if (!user->home)
-    {
-      if (chdir("/") < 0)
-	{
-	  werror("Strange: home directory was NULL, and chdir(\"/\") failed: %z\n",
-		 STRERROR(errno));
-	  return 0;
-	}
-    }
-  else if (chdir(user->home->data) < 0)
-    {
-      werror("chdir to %z failed (using / instead): %z\n",
-	     user->home ? (char *) user->home->data : "none",
-	     STRERROR(errno));
-      if (chdir("/") < 0)
-	{
-	  werror("chdir(\"/\") failed: %z\n", STRERROR(errno));
-	  return 0;
-	}
-    }
-  return 1;  
-}
-#endif
-
 struct lsh_string *
 format_userauth_failure(struct int_list *methods,
 					   int partial)
