@@ -1,6 +1,6 @@
 /* server_userauth.h
  *
- * $Id: server_userauth.h,v 1.10 2000/05/06 11:17:41 nisse Exp $
+ * $Id: server_userauth.h,v 1.11 2000/05/27 13:06:07 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -28,6 +28,7 @@
 #include "alist.h"
 #include "command.h"
 #include "connection.h"
+#include "io.h"
 #include "parse.h"
 #include "userauth.h"
 
@@ -52,7 +53,7 @@
 #define USER_LOOKUP(db, n, f) ((db)->lookup((db), (n), (f)))
 
 struct user_db *
-make_unix_user_db(int allow_root);
+make_unix_user_db(struct io_backend *backend, int allow_root);
 
 /* GABA:
    (class
