@@ -2,7 +2,7 @@
  *
  * Reads a sexp in given form from, and writes it in given form.
  *
- * $Id: sexp_conv.c,v 1.14 2000/02/22 21:46:05 nisse Exp $ */
+ * $Id: sexp_conv.c,v 1.15 2000/05/01 20:22:22 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -203,7 +203,7 @@ main_argp_parser(int key, char *arg, struct argp_state *state)
       }
     case OPT_HASH:
       {
-	int hash = lookup_hash(self->algorithms, arg, 0);
+	int hash = lookup_hash(self->algorithms, arg, NULL, 0);
 	if (hash)
 	  self->hash = hash;
 	else
