@@ -2,7 +2,7 @@
  *
  * User-related functions on UN*X
  *
- * $Id: unix_user.c,v 1.14 2000/05/08 20:17:26 nisse Exp $
+ * $Id: unix_user.c,v 1.18 2000/05/28 21:16:23 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -222,6 +222,8 @@ make_process_resource(pid_t pid, int signal)
      (super lsh_user)
      (vars
        (gid simple gid_t)
+       ; Needed for the USER_READ_FILE method
+       (backend object io_backend)
        
        ; These strings include a terminating NUL-character, for 
        ; compatibility with library and system calls. This applies also
@@ -300,6 +302,38 @@ do_file_exists(struct lsh_user *u,
   return 0;
 }
 
+/* NOTE: No arbitrary file names are passed to this function, so we don't have
+ * to check for things like "../../some/secret/file" */
+
+static void 
+do_read_file(struct lsh_user *u, 
+	     const char *name, int secret,
+	     struct command_continuation *c,
+	     struct exception_handler *e)
+{
+  CAST(unix_user, user, u);
+  struct lsh_string *f;
+  struct lsh_fd *fd;
+  const struct exception *x;
+  
+  if (!user->home)
+    {
+      EXCEPTION_RAISE(e, make_io_exception(EXC_IO_OPEN_READ, NULL,
+					   ENOENT, "No home directory"));
+      return;
+    }
+  
+  f = ssh_format("%lS/.lsh/%lz%c", user->home, name, 0);
+
+  fd = io_read_user_file(user->backend, f->data, user->super.uid, secret, &x, e);
+  lsh_string_free(f);
+
+  if (fd)
+    COMMAND_RETURN(c, fd);
+  else
+    EXCEPTION_RAISE(e, x);
+}
+
 /* Change to user's home directory. FIXME: If the server is running
  * as the same user, perhaps it's better to use $HOME? */
 
@@ -390,7 +424,12 @@ do_fork_process(struct lsh_user *u,
       if (log)
 	  /* FIXME: It should be safe to perform a blocking reverse dns lookup here,
 	   * as we have forked. */
+#if HAVE_UT_NAME
+	  logwtmp(log->ut_line, log->ut_name, log->ut_host);
+#elif HAVE_UT_USER
 	  logwtmp(log->ut_line, log->ut_user, log->ut_host);
+#endif
+
 #endif /* WITH_UTMP && HAVE_LOGWTMP */
       
       if (getuid() != user->super.uid)
@@ -509,6 +548,7 @@ do_exec_shell(struct lsh_user *u, int login,
 static struct lsh_user *
 make_unix_user(struct lsh_string *name,
 	       uid_t uid, gid_t gid,
+	       struct io_backend *backend,
 	       const char *passwd,
 	       const char *home,
 	       const char *shell)
@@ -520,6 +560,7 @@ make_unix_user(struct lsh_string *name,
   user->super.name = name;
   user->super.verify_password = do_verify_password;
   user->super.file_exists = do_file_exists;
+  user->super.read_file = do_read_file;
   user->super.chdir_home = do_chdir_home;
   user->super.fork_process = do_fork_process;
   user->super.exec_shell = do_exec_shell;
@@ -527,6 +568,8 @@ make_unix_user(struct lsh_string *name,
   user->super.uid = uid;
   user->gid = gid;
 
+  user->backend = backend;
+  
   /* Treat empty strings as NULL. */
 
 #define TERMINATE(s) (((s) && *(s)) ? format_cstring((s)) : NULL)
@@ -543,6 +586,7 @@ make_unix_user(struct lsh_string *name,
      (name unix_user_db)
      (super user_db)
      (vars
+       (backend object io_backend)
        (allow_root . int)))
 */
 
@@ -636,6 +680,7 @@ do_lookup_user(struct user_db *s,
   
       return make_unix_user(name,
 			    passwd->pw_uid, passwd->pw_gid,
+			    self->backend,
 			    crypted,
 			    passwd->pw_dir, passwd->pw_shell);
     }
@@ -648,11 +693,12 @@ do_lookup_user(struct user_db *s,
 }
 
 struct user_db *
-make_unix_user_db(int allow_root)
+make_unix_user_db(struct io_backend *backend, int allow_root)
 {
   NEW(unix_user_db, self);
 
   self->super.lookup = do_lookup_user;
+  self->backend = backend;
   self->allow_root = allow_root;
 
   return &self->super;
